import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  axisLeft,
  axisBottom,
  extent,
  area,
  curveBasis,
  bin,
  sum,
} from 'd3'; // ES6 syntax

const svg = d3.select('svg');

// dimensions
const width = +svg.attr('width'); // +  is parse
const height = +svg.attr('height');
const margin = {
  top: 50,
  right: 50,
  bottom: 70,
  left: 120,
};

const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const avgBMI = (v) =>
  sum(v, (d) => d.bodymassindex) / v.length;

// render function
const render = (data) => {
  const xValue = (d) => d.age;
  const yValue = (d) => d.bodymassindex;

  // scales
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth]);

  //console.log(data.map(xValue));
  // bin the data
  const binGenerator = bin()
    .value(xValue)
    .domain(xScale.domain());
  const binnedData = binGenerator(data);

  const yScale = scaleLinear()
    .domain(extent(binnedData, avgBMI))
    .range([innerHeight, 0])
    .nice();

  // 'frame' within 'canvas'
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left}, ${margin.top})`
    );

  // axes
  const xAxis = axisBottom(xScale)
    .ticks(14)
    .tickSize(`${-height + margin.top + margin.bottom + 3}`)
    .tickPadding(10);

  const yAxis = axisLeft(yScale)
    .ticks(5)
    .tickSize(`${-width + margin.left + margin.right}`)
    .tickPadding(10);

  // outputting axes
  const xAxisGroup = g
    .append('g')
    .call(xAxis)
    .attr(
      'transform',
      `translate(0, ${height - margin.top - margin.bottom})`
    );
  // formatting what axis details
  // are removed
  xAxisGroup.select('.domain').remove();

  // labels
  xAxisGroup
    .append('text')
    .attr('fill', '#8e8883')
    .style('font-size', '2rem')
    .attr(
      'x',
      `${(width - margin.left - margin.right) / 2}`
    )
    .attr('y', 60)
    .text('Age');

  const yAxisGroup = g.append('g').call(yAxis);

  yAxisGroup.select('.domain').remove();
  
  yAxisGroup
    .append('text')
    .attr('fill', '#8e8883')
    .style('font-size', '2rem')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .attr(
      'x',
      `${(-height + margin.top + margin.bottom) / 2}`
    )
    .attr('y', -70)
    .text('Average Body Mass Index');

  // Inspired by
  // https://observablehq.com/@d3/d3-group
  g.selectAll('rect')
    .data(binnedData)
    .join('rect')
    .attr('x', (v) => xScale(v.x0))
    .attr('y', (v) => yScale(avgBMI(v)))
    .attr('r', 10)
    .attr('width', (v) => xScale(v.x1) - xScale(v.x0))
    .attr(
      'height',
      (v) =>
        height -
        margin.top -
        margin.bottom -
        yScale(avgBMI(v))
    );

  
  // text
  g.append('text')
    .attr('y', -10)
    .text('BMI By Age')
    .style('font-size', '3rem')
    .attr('fill', '#635f5d');
};

// data importer
csv(
  'https://gist.githubusercontent.com/INDU1998-GIT/1c8f9495bca2a5a1d03ac6635b844827/raw/72e6d2f2a3f153a078fd96580beaa62ddbd022ef/age'
).then((data) => {
  data.forEach((d) => {
    d.exerany = +d.exerany;
    d.hlthplan = +d.hlthplan;
    d.smoke100 = +d.smoke100;
    d.height = +d.height;
    d.weight = +d.weight;
    d.wtdesire = +d.wtdesire;
    d.age = +d.age;
    d.bodymassindex =
      (+d.weight / d.height / d.height) * 703;
  });
  // render function call
  render(data);
});
